/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.classcreator.ClassGenerator;
import oued.persistance.configuration.PersistanceZeroConfigurator;
import oued.persistance.infosmodelgenerator.IInfosGenerator;
import oued.persistance.infosmodelgenerator.InfosGeneratorFactory;
import oued.persistance.infosmodelgenerator.XMIException;
import oued.persistance.mappingxml.HibernateConfigurationFile;
import oued.persistance.mappingxml.HibernateMappingFile;
import oued.persistance.mappingxml.CreateMappingFileException;
import oued.persistance.utils.UtilityClassFunction;

/**
 * La classe {@link GeneratorControl} encapsule toute la logique de gnration des diffrents fichiers : <br>
 * -Fichier cfg de configuration des pojo <br>
 * -Fichier hibernate.hbm.xml de configuration d'un projet eclipse <br>
 * -Fichier POJO
 * -Fichier DAO
 * @author OUEDRAOGO Moussa
 * @since 05-02-2007
 *
 */
public class GeneratorControl  {
	/**
	 * Le package des diffrents fichiers sources
	 */
	private String classPackage;
	
	/**
	 * Les packages des fichiers de ressources.
	 */
	private String ressourcesPackage="oued.ressources";
	
	/**
	 * Le repertoire de travail de persistance zero
	 */
	private String workingDir;
	
	/**
	 * Le repertoire des sources de persistance zero
	 */
	private String srcDir;
	
	
	/**
	 * Le constructeur personnalis  de {@link GeneratorControl}
	 * @param classPackage le le package des fichier sources  gnrer
	 * @param workingDir le repertoire de travail de persistance zero
	 * @param srcDir le repertoire des diffrents sources
	 */
	public GeneratorControl(String classPackage,String workingDir,String srcDir){
		this.classPackage=classPackage;			
		this.workingDir=workingDir;
		this.srcDir=srcDir;
	}
	
	public GeneratorControl(){
		
	}
	
	
	/**
	 * Cette mthode permet de compiler le fichier xmi. Elle cre et retourne le InfosModele
	 * neccessaire pour la gnration des fichiers sources et des fichiers de configuration
	 * @param xmiFile le nom complet du fichier xmi
	 * @return le InfosModele neccessaire pour la gnration des fichiers sources et des fichiers de configuration
	 * @throws XMIException Exception lance si une erreur survien lors du parsage du fichier XMI
	 */
	public InfosModel compileXMI(String xmiFile,String xmiVersion) throws XMIException{
		IInfosGenerator infosGenerator= InfosGeneratorFactory.getGenerator(xmiVersion);
		InfosModel infosModele=new InfosModel();
		//Cette methode lance la creation de l'infos modele
		infosGenerator.produceInfosModel(classPackage,xmiFile);	
		//Les diffrentes ressources  incorporer dans le infosModele
		Hashtable<String,Classe> classes = infosGenerator.getClasses();		
		Hashtable<String,Association> associations=infosGenerator.getAssociations();
		Hashtable<String,String> types=infosGenerator.getTypes();
		//Conserver les informations du modle gnres
		infosModele.setClasses(classes);
		infosModele.setAssociations(associations);
		infosModele.setTypes(types);
		return infosModele;
	}
	
	/**
	 * Cette mthode gnre les diffretes ressources correspondant au infosModele pass en paramtre
	 * @param infosModel le infosModele qui va servir  la gnration des diffrents fichier
	 * @param mappingConfigurator le mappingConfigurator qui contient les diffrents paramtres personnels de l'utilisateur
	 * @param genererFichierConfigClasse permet d'indiquer si on doit gnrer les fichiers de configuration des classes ou non
	 * @param genererClassePOJO permet d'indiquer si on doit gnrer les classes pojo ou non
	 * @param genererClasseDAO permet d'indiquer si on doit gnrer les classes dao ou non
	 * @param genererFichierConfigHibernate permet d'indiquer si on doit gnrer le fichier de configuration global d'hibernate ou non
	 * @throws PersistanceZeroException Exception qui est lance si une erreur survient  la gnration
	 */
	public void generate(InfosModel infosModel,PersistanceZeroConfigurator mappingConfigurator,boolean genererFichierConfigClasse,boolean genererClassePOJO,boolean genererClasseDAO,boolean genererFichierConfigHibernate) throws PersistanceZeroException{
		//Creer les repertoire des ressources et de package
		this.creerLesRepertoire();
		if (genererFichierConfigClasse){//On doit gnrer les fichiers de configuration des classes	
			//Creer les fichiers xml de configuration des classes				
			this.produceXMLs(mappingConfigurator,infosModel,this.classPackage,workingDir,this.srcDir);
		}
		
		//Creer les classes pojo et dao se trouvant dans le infosModele
		//Creer aussi les classes de ressources ncessaire au fonctionnement de ces dao et pojo
		ClassGenerator classGen=new ClassGenerator(infosModel);
		classGen.produceInfosModelClasses(workingDir,srcDir,genererClasseDAO,genererClassePOJO);
		classGen.produceRessourcesClasse(workingDir, srcDir, ressourcesPackage);
		
		if (genererFichierConfigHibernate){//On doit crer le fichier de configuration d'hibernate
			HibernateConfigurationFile hConfig=new HibernateConfigurationFile(this.classPackage,infosModel.getClasses(),workingDir,this.srcDir);
			hConfig.setMappingConfigurator(mappingConfigurator);
			hConfig.genererFichierConfiguration();
		}
	}
	
	
	
	
	public String getClassPackage() {
		return classPackage;
	}

	public void setClassPackage(String classPackage) {
		this.classPackage = classPackage;
	}

	public String getRessourcesPackage() {
		return ressourcesPackage;
	}

	public void setRessourcesPackage(String ressourcesPackage) {
		this.ressourcesPackage = ressourcesPackage;
	}

	public String getWorkingDir() {
		return workingDir;
	}

	public void setWorkingDir(String workingDir) {
		this.workingDir = workingDir;
	}

	public String getSrcDir() {
		return srcDir;
	}

	public void setSrcDir(String srcDir) {
		this.srcDir = srcDir;
	}

	/**
	 * Cette mthode cre les repertoire de ressources ainsi que les repertoire qui correspondent au package des dao et pojo
	 */
	private void creerLesRepertoire() throws PersistanceZeroException{
		//Creer les repertoires necessaires;
		try{
			String nomRep=this.workingDir+"\\"+this.srcDir+"\\"+UtilityClassFunction.replacePointBySlash(this.classPackage);
			File repertoirePackage=new File(nomRep);
			repertoirePackage.mkdirs();
			File repertoireRessources=new File(this.workingDir+"\\"+this.srcDir+"\\"+UtilityClassFunction.replacePointBySlash(this.ressourcesPackage));
			repertoireRessources.mkdirs();
		}catch(Exception e){
			throw new PersistanceZeroException("Un problme inconnu est survenu lors de la cration des repertoires");
		}
	}
	
	
	/**
	 * Methode permettant de generer l'ensemble des fichiers XML correspondant aux diffrentes classes repertories.
	 *
	 */
	public void produceXMLs(PersistanceZeroConfigurator mappingConfigurator,InfosModel infosModel,String pojoPackage,String repertoirePersitanceZero,String repertoireSources){
		DocumentBuilder constructeur;
		DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
		try {
			constructeur = fabrique.newDocumentBuilder();			
			Enumeration<Classe>  it=infosModel.getClasses().elements();
			while (it.hasMoreElements()){
				Classe classe=it.nextElement();			
				HibernateMappingFile mappingClasse=new HibernateMappingFile(infosModel,classe);
				mappingClasse.setConstructeur(constructeur);
				//La classe ne doit pas etre une classe fille ou une classe association.
				//En effet ces deux types de classes dans nos cas n'ont pas de fichier xml
				if (classe.getNom().compareTo("Personnel")==0){
					System.out.println("je suis al");
				}
				if (!classe.isChild() && !classe.isEstClasseAssociation()){
					try {
						//Recuperer le mapping Configurator pour appliquer les proprietes personnalisees, s'il yen a
						mappingClasse.setClasseConfigurator(mappingConfigurator.getClasseConfigurator(classe.getNom()));
						mappingClasse.createMappingXML(workingDir,srcDir);
					} catch (CreateMappingFileException e) {
						System.out.println("PersistanceZero:Probleme lors de la cration de la classe "+classe.getNom()+". "+e.getMessage());
					}
				}
			}
			
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		
	}
	
	
}
