/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance;

import java.io.File;
import java.io.FileNotFoundException;

import oued.persistance.classconcepts.InfosModel;
import oued.persistance.configuration.PersistanceZeroConfigurator;
import oued.persistance.infosmodelgenerator.InfosGeneratorFactory;
import oued.persistance.infosmodelgenerator.XMIException;


/**
 * Classe principale du projet PersistanceZero.
 * PersistanceZero permet de gnrer  partir d'un modle de classe,
 * l'ensemble des fichiers de persistance. Ces fichiers sont : <br>
 * -Les classes appeles POJO (Plain Old Java Object)
 * -Les fichiers de configuration des classes afin de pouvoir s'interfacer  la base de donnes
 * -Le fichier hibernate.hbm.cfg de configuration du projet
 * -La base de donnes elle mme
 * @author OUEDRAOGO Moussa
 * @since 04-02-2007
 *
 */
public class PersistanceZero {
	GeneratorControl genControl=null;
	/**
	 * InfosModel contenant les diffrentes ressources issues du fichier XMI au d'une autre source de donnes
	 */
	private InfosModel infosModel=null;
	
	/**
	 * le MappingConfigurator par  dfaut. Pour rappel le mapping configurator regroupe l'ensemble des paramtres
	 * personnaliss de l'utilisateur
	 */
	private PersistanceZeroConfigurator mappingConfigurator=null;
	
	/**
	 * Le chemin complet du fichier XMI  utiliser pour la gnration
	 */
	private String xmiFile;
	
	/**
	 * La version du fichier XMI. Par dfaut la version utilise est 1.3
	 */
	private String xmiVersion="1.3";
	
	/**
	 * Constantes reprsentant les diffrentes versions du fichier XMI
	 */
	public static String XMI_2_1=InfosGeneratorFactory.XMI_2_1;
	
	
	/**
	 * Constructeur personnalis de la classe {@link PersistanceZeroException}
	 * @param workingDir le repertoire de travail de persistance zero. C'est dans ce repertoire que sera gnr tous les fichiers
	 * @param srcDir Repertoire des .java gnr. ce repertoire est relatif au repertoire de travail
	 * @param createIfNotExist Si vrai, recr le projet s'il existe.Dans le cas  contraire, il soulve un exception FichierInexistantException
	 * @throws FichierInexistantException
	 */
	public PersistanceZero(String workingDir,String srcDir,boolean createIfNotExist) throws FileNotFoundException{
		 genControl=new GeneratorControl(null,workingDir,srcDir);
		this.createProject(workingDir,srcDir, createIfNotExist);
	}
	
	/**
	 * Crer l'ensemble des ressources de la couche de persistance <br>
	 * (Classes pojo, classe DAO, fichiers de configuration xml)
	 * @throws PersistanceZeroException 
	 */
	public void process() throws IllegalStateException,PersistanceZeroException{
		if (infosModel==null)
			throw new IllegalStateException("Vous devez compilez  PersistanceZero. Les informations sur le modle semble tre incorrectes");
		this.buildRessources(true,true,true,true);
	}
	
	/**
	 * Dfinir le fichier XMI qui va servir  la gnration des classes 
	 * @param xmiFile le fichier XMI
	 * @throws FichierInexistantException cette exception est lance si le fichier n'existe pas
	 */
	public void defineXmiFile(String xmiFile,String xmiVersion) throws FileNotFoundException{
		//Le repertoire de travail doit tre dfini avant tout autre opration.Lancez une exception
		if (genControl.getWorkingDir()==null)
			throw new IllegalStateException("Le rpertoire de travail n'est pas dfini.");
		//Si le nom du fichier est incorrect, lancez une exception
		if (xmiFile==null || xmiFile.trim().compareTo("")==0)
			throw new IllegalArgumentException("le nom du fichier ne peut tre une chaine nulle ou vide");		
		//Si la version du fichier xmi est incorrecte, lancez une exception
		if (xmiVersion==null || xmiVersion.trim().compareTo("")==0)
			throw new IllegalArgumentException("la version du fichier xmi ne peut tre une chaine nulle ou vide");		
		File file=new File(xmiFile);
		//Si le fichier XMI n'existe pas sur le disque, lancez une exception
		if (!file.exists()) throw new FileNotFoundException("Le fichier XMI n'existe pas");
		this.xmiVersion=xmiVersion;
		this.xmiFile=xmiFile;
		
	}
	
	/**
	 * Cette mthode permet de dfinir les ressources qui vont servir  la gnration des classes
	 * @param infosModel la classe qui contient les diffrentes ressources <br>
	 * (les diffrentes classes, les diffrentes associations, les diffrents types). Ces informations sont obtenues
	 * le plus souvent d'un fichier XMI. 
	 */
	public void defineInfosModel(InfosModel infosModel){
		//InfosModele est null. Lancez une exception
		if (infosModel==null)
			throw new IllegalArgumentException("InfosModele ne peut tre null");
		//Une des ressources essentielles de l'InfosModele est nulle. Lancez une exception
		if (infosModel.getClasses()==null
				|| infosModel.getAssociations()==null
				|| infosModel.getTypes()==null)
			throw new IllegalArgumentException("Les informations sur le modle (classes,associations,types) ne doivent tre nulles");
		
		this.infosModel=infosModel;
	}
	
	/**
	 * Cette mthode permet de gnrer les fichiers de configuration
	 * des classes du modele. A titre d'exemple, si nous avons une classe Etudiant,
	 * le fichier Etudiant.hbm.xml sera gnr
	 * @throws PersistanceZeroException Exception lance si aucun infosModele n'est dfini
	 */
	public void generateXmlFiles() throws  PersistanceZeroException{
		//InfosModele est null. Lancez une exception
		if (infosModel==null)
			throw new IllegalStateException("Vous devez d'abord compiler persistanceZero. Les informations sur le modle semble tre incorrect");
		this.buildRessources(true,false,false,false);
	}
	
	/**
	 * Cette mthode permet de gnrer les classes de notre modle
	 * @throws PersistanceZeroException Exception lance si aucun infosModele n'est dfini
	 */
	public void generateClassFiles() throws PersistanceZeroException{
		if (infosModel==null)
			throw new IllegalStateException("Vous devez d'abord compiler persistanceZero. Les informations sur le modle semble tre incorrect");
		this.buildRessources(false,true,false,false);
	}
	
	/**
	 * Cette mthode permet de gnrer les classes DAO (Data Access Object )
	 * @throws PersistanceZeroException Exception lance si aucun infosModele n'est dfini
	 */
	public void generateDaoFiles() throws PersistanceZeroException{
		if (infosModel==null)
			throw new IllegalStateException("Vous devez compilez d'abord PersistanceZero. Les informations sur le modle semble tre incorrect");
		this.buildRessources(false,false,true,false);
	}
	
	/**
	 * Gnrer le fichier de configuration global d'hibernate hibernate.hbm.cfg
 * @throws PersistanceZeroException Exception lance si aucun infosModele n'est dfini
	 * @throws XMIException 
	 */
	public void generateHibernateConfigFile() throws PersistanceZeroException{
		if (infosModel==null)
			throw new IllegalStateException("Vous devez compilez d'avoir PersistanceZero. Les informations sur le modle semble tre incorrect");
		this.buildRessources(false,false,false,true);
	}
	
	/**
	 * Cette mthode permet de compiler le fichier XMI afin de s'assurer qu'il est correcte.
	 * Cela a pour consquence de crer un InfosModele que l'on utilisa pour la cration de nos diffrentes ressources
	 * @throws XMIException exception lance lorsque le fichier XMI est incorrect ou corrompu
	 */
	public void compileXmi() throws XMIException{
		infosModel=genControl.compileXMI(xmiFile,this.getXmiVersion());
	}
	

	/**
	 * Cette methode retourne le chemin du fichier XMI utilis pour la cration des ressources
	 * @return le fichier XMI utilis pour la cration des diffrentes ressources
	 */
	public String getXmiFile() {
		return xmiFile;
	}

	
	/***
	 * Cette mthode retourne le package des sources gnres
	 * @return le package des sources gnres
	 */
	public String getClassPackage() {
		return genControl.getClassPackage();
	}

	/**
	 * Cette mthode permet de modifier le package des sources gnres
	 * @param packageCouchePersistance le nouveau package
	 */
	public void setClassPackage(String packageCouchePersistance) {
		this.genControl.setClassPackage(packageCouchePersistance);
	}

	/**
	 * Cette mthode retourne la version  du fichier XMI pour lequel il faut effectuer la gnration
	 * @return  la version  du fichier XMI pour lequel il faut effectuer la gnration
	 */
	public String getXmiVersion() {
		return xmiVersion;
	}

	/**
	 * Cette mthode retourne L'InfosModele qui contient l'ensemble des ressources pour la gnration
	 * @return   L'InfosModele qui contient l'ensemble des ressources pour la gnration
	 */
	public InfosModel getInfosModel() {
		return infosModel;
	}

	/**
	 * Cette mthode retourne le repertoire de persistance (repertoire de travail)
	 * @return   le repertoire de persistance (repertoire de travail)
	 */
	public String getRepertoirePersitanceZero() {
		return genControl.getWorkingDir();
	}

	/**
	 * 
	 * @param workingDir le repertoire de travail de persitanceZero
	 * @param createIfNotExist si dfini a vrai, cre le repertoire s'il n'existe pas. Dans le cas contraire
	 * lance une exception FichierInexistantException 
	 * @throws FichierInexistantException exception lance si le fichier n'existe pas et creerSiInexistant=false
	 */
	public void createProject(String workingDir,String srcDir,boolean createIfNotExist)throws FileNotFoundException{
		File directory=new File(workingDir);
		if (!createIfNotExist){ //creerSiInexistant=false, lancer une exception si le fichier n'existe pas
			if (!directory.exists()) throw new FileNotFoundException("Le fichier XMI n'existe pas");
		}
		//On creer le repertoire de travail
		directory.mkdirs();
		
		//On creer les repertoire des sources
		this.genControl.setClassPackage(srcDir);
		if (this.genControl.getClassPackage().trim().compareTo("")!=0){
			File repertoireSource=new File(workingDir+"\\"+srcDir);
			repertoireSource.mkdirs();
			
		}
		
		this.genControl.setWorkingDir(workingDir);
		this.genControl.setClassPackage(srcDir);
	}
	

	/**
	 * Cette  mthode permet de crer les diffrentes ressources.
	 * @param genererFichierConfigClasse  vrai si l'on veut gnrer les fichiers de configuration, et faux sinon
	 * @param genererClassePOJO  vrai si l'on veut gnrer les classes POJO, et faux sinon
	 * @param genererClasseDAO  vrai si l'on veut gnrer les diffrents DAO et faux sinon
	 * @param genererFichierConfigHibernate  vrai si l'on veut  crer le fichier de configuration hibernate et faux sinon
	 * @throws PersistanceZeroException Exception lance lorsqu'une erreur se produit lors de la construction
	 */
	private void buildRessources(boolean genererFichierConfigClasse,boolean genererClassePOJO,boolean genererClasseDAO,boolean genererFichierConfigHibernate)  throws PersistanceZeroException{
		if (mappingConfigurator==null)
			this.setPZeroConfigurator(new PersistanceZeroConfigurator());
		genControl.generate(infosModel,mappingConfigurator,genererFichierConfigClasse,genererClassePOJO,genererClasseDAO,genererFichierConfigHibernate);		
	}
	
	/**
	 * Cette mthode retourne le repertoire des sources du projet. Ce repertoire  est relatif
	 * au repertoire de travail 
	 * @return le repertoire des sources du projet. Ce repertoire  est relatif
	 */
	public String getRepertoireSource() {
		return genControl.getClassPackage();
	}

	/**
	 * Cette mthode permet de modifier le repertoire des sources du projet
	 * @param repertoireSource le nouveau repertoire des sources
	 */
	public void setRepertoireSource(String repertoireSource) {
		this.genControl.setClassPackage( repertoireSource);
	}
	
	/**
	 * Cette mthode permet de dfinir MappingConfigurator  utiliser pour les paramtres dfinis
	 * par l'utilisateur
	 * @param mappingConfigurator le MappingConfigurator  utiliser
	 */
	public void setPZeroConfigurator(PersistanceZeroConfigurator mappingConfigurator){
		this.mappingConfigurator=mappingConfigurator;
	}
	
}