/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.configuration;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Cette Classe permet d'encapser tous les paramtres personnaliss concernant
 * une classe. 
 * @author OUEDRAOGO Moussa
 * @since 05-05-2007
 *
 */
public class ClassConfigurator {
	/**
	 * Le nom de la classe  qui est destin ces paramtres
	 */
	private String nomClasse;	
	
	/**
	 * La liste des Configuration de cette classe
	 */
	private List<Configurator> configurations;
	
	/**
	 * Constructeur personnalis
	 * @param nomClasse le nom de la classe
	 */
	public ClassConfigurator(String nomClasse){
		this.nomClasse=nomClasse;
		this.configurations=new ArrayList<Configurator>();
	}
	
	/**
	 * dfini les  paramtres de l'utilisateur pour le context context
	 * @param context le context ou se dfini ces paramtres
	 * @param parametres les diffrents paramtres
	 */
	public void define(String context,String parametres){
		if (context.compareTo(PersistanceZeroConfigurator.CLASS)!=0
				&& context.compareTo(PersistanceZeroConfigurator.VERSION)!=0
				&& context.compareTo(PersistanceZeroConfigurator.ID)!=0) 
			throw new IllegalArgumentException("Le context doit etre gal  CLASS_ATTRIBUT ou VERSION_ATTRIBUT ou ID_CONTEXT lorsqu'aucun attribut n'est prcis");
		
		this.internalDefine(context, null, parametres);
		
	}
	
	/**
	 * dfini les  paramtres de l'utilisateur pour le context context
	 * @param context le context ou se dfini ces paramtres
	 * @param nomAttribut le nom de l'attribut sur lequel les paramtres sont appliqus
	 * @param parametres les diffrents paramtres
	 */
	public void define(String context,String nomAttribut,String parametres){
		if (context.compareTo(PersistanceZeroConfigurator.CLASS)==0
				|| context.compareTo(PersistanceZeroConfigurator.VERSION)==0)			
			throw new IllegalArgumentException("Le context doit etre diffrent de CLASS_ATTRIBUT et  VERSION_ATTRIBUT et ID_CONTEXT lorsqu'un attribut est prcis");
		
		this.internalDefine(context, nomAttribut, parametres);
	}
	
	/**
	 * retourne le nom de la classe 
	 * @return le nom de la classe 
	 */
	public String getNomClasse() {
		return nomClasse;
	}

	/**
	 * modifie le nom de la classe 
	 * @param nomClasse le nouveau nom de la classe 
	 */
	public void setNomClasse(String nomClasse) {
		this.nomClasse = nomClasse;
	}
	
	/**
	 * retourne le {@link Configurator} qui a t dfini pour l'attribut nomAtribut
	 * @param nomAttribut le nom de l'attribut pour lequel on cherche son configuration
	 * @return le {@link Configurator} qui a t dfini pour l'attribut nomAtribut
	 */
	public Configurator getAttributConfiguration(String nomAttribut){
		Iterator<Configurator>it=configurations.iterator();
		while (it.hasNext()){
			Configurator config=it.next();
			if (config.getNomAttribut()==null) continue;
			if (config.getNomAttribut().compareTo(nomAttribut)==0)
				return config;
		}
		return null;
	}
	
	/**
	 * retourne le premier {@link Configurator} rencontr pour le context pass en paramtre
	 * @param context le context
	 * @return le premier {@link Configurator} rencontr pour le context pass en paramtre
	 */
	public Configurator getFirstContextConfiguration(String context){
		Iterator<Configurator>it=configurations.iterator();
		while (it.hasNext()){
			Configurator config=it.next();				
			if (config.getMappingContext().getContext().compareTo(context)==0)
				return config;
		}
		return null;
	}
	
	/**
	 * retourne tous les {@link Configurator} correspondant au contexte context
	 * @param context le context pourlequel on cherche ses configurations
	 * @return tous les {@link Configurator} correspondant au contexte context
	 */
	public List<Configurator> getAllContextConfiguration(String context){
		List<Configurator> listeTemp=new ArrayList<Configurator>();
		Iterator<Configurator>it=configurations.iterator();
		while (it.hasNext()){
			Configurator config=it.next();				
			if (config.getMappingContext().getContext().compareTo(context)==0){
				listeTemp.add(config);
			}
		}
		return listeTemp;
	}
	
	/**
	 * retourne tous les {@link Configurator} correspondant qui appartient  au moins un contexte
	 * de contexts 
	 * @param contexts la liste des contextes
	 * @return tous les {@link Configurator} correspondant qui appartient  au moins un contexte
	 * de contexts
	 */
	public List<Configurator> getAllContextConfiguration(List<String> contexts){
		List<Configurator> listeTemp=new ArrayList<Configurator>();
		Iterator<Configurator>it=configurations.iterator();
		while (it.hasNext()){
			Configurator config=it.next();
			Iterator<String> itContext=contexts.iterator();
			while(itContext.hasNext()){
				String context=itContext.next();
				if (config.getMappingContext().getContext().compareTo(context)==0){
					listeTemp.add(config);
					continue;
				}
			}
		}
		return listeTemp;
	}
	
	/**
	 * Cre un Configuration correspondant aux paramtres utilisateur
	 * @param context le context ou est dfini les paramtres
	 * @param nomAttribut le nom de l'attribut proprits de cette {@link Configurator}
	 * @param parametres les paramtres
	 */
	private void internalDefine(String context,String nomAttribut,String parametres){
		Configurator configuration=new Configurator();
		configuration.setMappingContext(ContextFactory.getContext(context));
		
		if (configuration.getMappingContext()==null) throw new IllegalArgumentException("Le nom de context est incorrect");
		
		configuration.setNomAttribut(nomAttribut);
		configuration.setParametres(parametres);
		this.configurations.add(configuration);
		
	}
}
